@@ -2,37 +2,36 @@
  *
  * Copyright (C) 2008-2011 Red Hat, Inc.
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __G_NETWORKING_H__
 #define __G_NETWORKING_H__
 
 #include <glib.h>
+#include <gio/gio-visibility.h>
 
 #ifdef G_OS_WIN32
-
-#ifndef _WIN32_WINNT
-#define _WIN32_WINNT 0x0501
-#endif
 #include <winsock2.h>
-#undef interface
 #include <ws2tcpip.h>
 #include <windns.h>
 #include <mswsock.h>
-
+#include <wspiapi.h>
 #include <iphlpapi.h>
+#undef interface
 
 #else /* !G_OS_WIN32 */
 
@@ -74,7 +73,7 @@
 
 G_BEGIN_DECLS
 
-GLIB_AVAILABLE_IN_2_36
+GIO_AVAILABLE_IN_2_36
 void g_networking_init (void);
 
 G_END_DECLS
